// DARCIIConfig.h
// Data structure used to describe DARC-II behaviour

#define MAXNAMELENGTHINCZ 17
#define MAXPINLENGTHINCZ 9

// where to store structure in Toothpick
#define DARCII_CONFIG_LOC  0x00FE00

// structure definition
typedef struct
{
//0xFE00
	unsigned char Initialized;		// equal to 0x55 if this structure contains valid data
	unsigned char ServerName[MAXNAMELENGTHINCZ];
	unsigned char PINCode[MAXPINLENGTHINCZ];
//0xFE1B
	unsigned char SecurityLevel;	// 0 = none, 1 = PIN code, 2 = PIN and encryption
	unsigned char ResetOnError;		// nonzero if DARC should reset on error
	unsigned char nAnalogChannel;	// number of analog channels
	unsigned char IsRefAN2;			// if nonzero AN2 is -ve reference 
	unsigned char IsRefAN3;			// if nonzero AN3 is +ve reference 
//0xFE20
	unsigned char PinFuncAN0;		// 0 = input, 1 = output; ignored if analog 
	unsigned char PinFuncAN1;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN2;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN3;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN4;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN5;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN6;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN7;		// 0 = input, 1 = output; ignored if analog
//0xFE28
	unsigned char PinFuncAN8;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN9;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN10;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncAN11;		// 0 = input, 1 = output; ignored if analog
	unsigned char PinFuncCCP1;		// 0 = input, 1 = output, 2 = PWM output
	unsigned char PinFuncCCP2;		// 0 = input, 1 = output, 2 = PWM output
	unsigned char PinFuncCCP3;		// 0 = input, 1 = output, 2 = PWM output
	unsigned char PinFuncCCP4;		// 0 = input, 1 = output, 2 = PWM output
//0xFE30
	unsigned char PinFuncCCP5;		// 0 = input, 1 = output, 2 = PWM output
	unsigned char PinFuncINT0;		// 0 = input, 1 = output
	unsigned char PinFuncINT1;		// 0 = input, 1 = output
	unsigned char PinFuncTxD;		// 0 = input, 1 = output
	unsigned char PinFuncRxD;		// 0 = input, 1 = output
	unsigned char PinFuncSDA;		// 0 = input, 1 = output
	unsigned char PinFuncSCL;		// 0 = input, 1 = output
	unsigned char PinFuncSDO;		// 0 = input, 1 = output
//0xFE38
	unsigned char ParallelA;		// Number of bits in parallel I/O; input or output must be specified separately
	unsigned char ParallelB;		// Number of bits in parallel I/O; input or output must be specified separately
	unsigned char ParallelC;		// Number of bits in parallel I/O; input or output must be specified separately
	unsigned char TimeBase;			// 0xFF = no PWM, 0 = 0.2us, 1 = 0.8us, 2 = 3.2us
	unsigned char PWMPeriod;		// PWM period is (PWMPeriod+1) TimeBase units
	unsigned char RefreshRate;		// 0 = Never, 1 = 100ms, 2 = 200ms, 3 = 500ms, 4 = 1s, 5 = 2s, 6 = 5s, 7 = 10s, 8 = 15s, 9 = 30s, 10 = 1min, 11 = 2min, 12 = 5min, 13 = 10min, 14 = 15min, 15 = 30min, 16 = 60min
//0xFE3E
	unsigned char ToFromPin[256];	// Pin associated with control with ID equal to index number - see below
//0xFF3E
	unsigned char Dummy[0x100-0x3E];	// dummy
}
DARCIIConfig;

// ToFromPin definitions.  First the info in a table, then as macros

// ----------------------------------------------------------------------------------------
// Value | Button/Image/Latch/Text |           List             |   Number   |  DT Matrix | 
// ----------------------------------------------------------------------------------------
//   0   |           None          |           None             |    None    |    None    |  
//   1   |           AN0           |        Parallel A          |    AN0     |    AN0     |   
//   2   |           AN1           |        Parallel B          |    AN1     |    AN1     |   
//   3   |           AN2           |        Parallel C          |    AN2     |    AN2     |  
//   4   |           AN3           |           CCP1             |    AN3     |    AN3     |  
//   5   |           AN4           |           CCP2             |    AN4     |    AN4     | 
//   6   |           AN5           |           CCP3             |    AN5     |    AN5     |
//   7   |           AN6           |           CCP4             |    AN6     |    AN6     |
//   8   |           AN7           |           CCP5             |    AN7     |    AN7     |
//   9   |           AN8           |           None             |    AN8     |    AN8     |
//   10  |           AN9           |           None             |    AN9     |    AN9     |
//   11  |           AN10          |           None             |    AN10    |    AN10    |
//   12  |           AN11          |           None             |    AN11    |    AN11    |
//   13  |           CCP1          |           None             | Parallel A | Parallel A |
//   14  |           CCP2          |           None             | Parallel B | Parallel B |
//   15  |           CCP3          |           None             | Parallel C | Parallel C |
//   16  |           CCP4          |           None             |    CCP1    |    None    |
//   17  |           CCP5          |           None             |    CCP2    |    None    |
//   18  |           INT0          |           None             |    CCP3    |    None    |
//   19  |           INT1          |           None             |    CCP4    |    None    |
//   20  |           TxD           |           None             |    CCP5    |    None    |
//   21  |           RxD           |           None             |    None    |    None    |
//   22  |           SDA           |           None             |    None    |    None    |
//   23  |           SCL           |           None             |    None    |    None    |
//   24  |           SDO           |           None             |    None    |    None    |
// ----------------------------------------------------------------------------------------

// Button/Image/Latch/Text macros
#define TFP_BILT_None	0
#define TFP_BILT_AN0	1
#define TFP_BILT_AN1	2
#define TFP_BILT_AN2	3
#define TFP_BILT_AN3	4
#define TFP_BILT_AN4	5
#define TFP_BILT_AN5	6
#define TFP_BILT_AN6	7
#define TFP_BILT_AN7	8
#define TFP_BILT_AN8	9
#define TFP_BILT_AN9	10
#define TFP_BILT_AN10	11
#define TFP_BILT_AN11	12
#define TFP_BILT_CCP1	13
#define TFP_BILT_CCP2	14
#define TFP_BILT_CCP3	15
#define TFP_BILT_CCP4	16
#define TFP_BILT_CCP5	17
#define TFP_BILT_INT0	18
#define TFP_BILT_INT1	19
#define TFP_BILT_TxD	20
#define TFP_BILT_RxD	21
#define TFP_BILT_SDA	22
#define TFP_BILT_SCL	23
#define TFP_BILT_SDO	24

// List macros
#define TFP_L_None		0
#define TFP_L_ParallelA	1	
#define TFP_L_ParallelB	2	
#define TFP_L_ParallelC	3	
#define TFP_L_CCP1		4	
#define TFP_L_CCP2		5
#define TFP_L_CCP3		6	
#define TFP_L_CCP4		7	
#define TFP_L_CCP5		8	


// Number / DT Matrix macros
#define TFP_NDTM_None		0
#define TFP_NDTM_AN0		1
#define TFP_NDTM_AN1		2
#define TFP_NDTM_AN2		3
#define TFP_NDTM_AN3		4
#define TFP_NDTM_AN4		5
#define TFP_NDTM_AN5		6
#define TFP_NDTM_AN6		7
#define TFP_NDTM_AN7		8
#define TFP_NDTM_AN8		9
#define TFP_NDTM_AN9		10
#define TFP_NDTM_AN10		11
#define TFP_NDTM_AN11		12
#define TFP_NDTM_ParallelA	13	
#define TFP_NDTM_ParallelB	14
#define TFP_NDTM_ParallelC	15	
#define TFP_NDTM_CCP1		16	// for Number control only
#define TFP_NDTM_CCP2		17	// for Number control only
#define TFP_NDTM_CCP3		18	// for Number control only
#define TFP_NDTM_CCP4		19	// for Number control only
#define TFP_NDTM_CCP5		20	// for Number control only



