//------------------------------------------------------------------------------------------------------
//
//  Toothpick.c - Toothpick settings 
//
//  You will need a copy of this for each application you develop,
//  so copy it from the Development Kit directory to your application
// development directory and modify it there
//
//  (c) FlexiPanel Ltd
//
//-------------------------------------------------------------------------------------------------------
 
#define __Toothpick_c__
#include "Toothpick.h"
#include <p18cxxx.h>

// If programming a FlexiPanel UI from FlexiPanel Designer using data files, include the header
// file Designer creates, (comment out otherwise)
#include "HappyTerminalRes.h" 

//-----------------------------------------------------------------------------------------

// Toothpick settings  
#pragma romdata TOOTHPICK_SETTINGS=0x00C080

//-------------------------sec-min-hour-date-dow-month-year
rom DateTimeU RESETTIME = {	00, 00, 00,  01,  6,   01, 2000 };
rom unsigned char DSTDEFAULT = DST_NONE;

// ToothpickSettings flags - OR together TPS_values 
// TPS_RealTimeClock	- implement real time clock
// TPS_ClientTick	  	- send tick event messages to LowInterrupt
rom unsigned char ToothpickSettings = TPS_RealTimeClock | TPS_ClientTick  ;

/* You can change the name (up to 64 characters) but do NOT change the maximum length LOCALNAMELEN*/
/* The name must be ASCII.   You can also change the name at runtime by writing */
/* new name here (remember the zero terminator). Change takes effect after reset, */
/* and many remote devices won't notice if the device has already been discovered before */
/* The DESIGNERCHOOSESNAME overrides the name during development (with Data */
/* file target) when the Force layout flag is set in Designer */
#ifndef DESIGNERCHOOSESNAME
rom unsigned char pLocalName[LOCALNAMELEN] = "Toothpick";    
#else
rom unsigned char pLocalName[LOCALNAMELEN] = DESIGNERCHOSENNAME;    
#endif

#if !defined( __18F67J10 )
/* Toothpick will set this value to the attached LinkMatik's MAC address, highest byte first (00, 07, 80...)  */
rom unsigned char pLocalBTAddr[BTADDRLEN]; 

/* Toothpick will set this to a zero terminated ASCII software version number*/
rom unsigned char pVersionStr[VERSTRLEN]; 
#endif

/* You can set the authentication PIN to what you want. You can also change the PIN at */
/* runtime by writing a new zero terminated ASCII PIN here and resetting (max 16 ASCII characters plus */
/* zero terminator).  Bear in mind some devices (phones) can only enter PIN codes with */
/* digits 0-9.  For no PIN, set the pszPIN to "".  No pin is not a good idea since many operating systems */
/* including Windows may refuse to connect to it on grounds of security, don't ask us why */
rom char pszPIN[SZPINMAX] = "0000";

/* Bluetooth class must be specified by a 6 hex digit string.  Refer to documentation for details of determining device class values */
rom char pBTClass[] = "522204";   // Hex digits MUST be in lower case (a-f) otherwise initialization will fail 

/* Bluetooth pagemode sets whether the device is discoverable and/or connectable. First digit is as follows: */
/* 0 = not discoverable, not connectable */
/* 1 = discoverable, not connectable */
/* 2 = not discoverable, connectable */
/* 3 = discoverable, connectable */
/* 4 = discoverable, connectable if not connected; not discoverable, not connectable if connected */
rom char pPageMode[] = "4 2000 1";  // Hex digits MUST be in lower case (a-f) otherwise initialization will fail

// LinkMatik SET CONTROL CONFIG settings, refer to LinkMatik docs for details
rom char pLMTConfig[] = "a1";  // Hex digits MUST be in lower case (a-f) otherwise initialization will fail
 
// TxBlockSize is the maximum number of bytes of data that get transmitted per Bluetooth frame
// Recommended value right now is 100.  We should be able to increase it to 255 or 511 later, 
// pending further info from the LinkMatik firmware developers
rom unsigned short TxBlockSize = 100 ;

#pragma romdata 

//-----------------------------------------------------------------------------------------
// LinkMatik receive buffer

#define LMTRXBUFFLEN 256		// do NOT change - circular buffer assumes length of 256
#define LMTRXSOURCELEN 64		// do NOT change - circular buffer assumes length of 256

#pragma udata LMTRXSOURCE
volatile unsigned char LMTRXSource[LMTRXSOURCELEN];
#pragma udata LMTRXBUFF
volatile unsigned char LMTRxBuff[LMTRXBUFFLEN];
#pragma udata 

//-----------------------------------------------------------------------------------------

// public RAM data definitions; data larger than char are double word aligned
#pragma udata TOOTHPICKD
volatile DateTimeU 		RealTimeClock;				// Real Time Clock, seconds to years
volatile unsigned char 	DSTEvent;			  		// Daylight Savings Time rule in effect
volatile unsigned char 	ToothpickSemaphores;  		// OS status semaphores
volatile unsigned char 	ToothpickSemaphores2;  		// OS status semaphores
volatile unsigned char 	SWIflags;  					// event(s) which triggered software interrupt
volatile unsigned char 	FxPChannel;					// FlexiPanel client logical channel
volatile unsigned char 	NumConnections;				// Number of devices connected
volatile unsigned char 	LMTRxStart;					// Receive buffer startpoint
volatile unsigned char 	LMTRxEnd;					// Receive buffer endpoint
volatile unsigned char 	pRemoteBTAddr[BTADDRLEN]; 	// bluetooth address of last remote device to connect 
#if defined( __18F67J10 )
unsigned char pLocalBTAddr[BTADDRLEN]; 				/* Toothpick will set this value to the attached LinkMatik's MAC address, highest byte first (00, 07, 80...)  */
unsigned char pVersionStr[VERSTRLEN]; 				/* Toothpick will set this to a zero terminated ASCII software version number*/
#endif
unsigned char usDelayCounter; 	

//--------------------------------------------------------------------------------------------------------

// Configuration bits - Change them if you want, at your own risk

#if !defined( __18F67J10 )
    #pragma romdata CONFIG1H = 0x300001
    const rom unsigned char config1H = 0b00000010;     	// HS oscillator

#if defined( __18F6722 )
    #pragma romdata CONFIG2L = 0x300002
    const rom unsigned char config2L = 0b00011111;      // Brown-out Reset Enabled in hardware @ 2.5V, PWRTEN disabled
#elif defined( __18F6720 )
    #pragma romdata CONFIG2L = 0x300002
    const rom unsigned char config2L = 0b00001111;      // Brown-out Reset Enabled in hardware @ 2.5V, PWRTEN disabled
#else
error
#endif 

    #pragma romdata CONFIG2H = 0x300003
    const rom unsigned char config2H = 0b00001110;      // HW WD disabled, 1:128 prescaler

#if defined( __18F6722 )

#elif defined( __18F6720 )
    #pragma romdata CONFIG3L = 0x300004
    const rom unsigned char config3L = 0b11111111;      // Microcontroller mode
#else
error
#endif

    #pragma romdata CONFIG3H = 0x300005
    const rom unsigned char config3H = 0b10000000;      // CCP2 mux RE7

    #pragma romdata CONFIG4L = 0x300006
#ifdef Enable_Debug
    const rom unsigned char config4L = 0b00000001;      // DEBUG enabled,
                                                        // LVP disabled
                                                        // STVREN enabled
#else
    const rom unsigned char config4L = 0b10000001;      // DEBUG disabled,
                                                        // LVP disabled
                                                        // STVREN enabled
#endif
    #pragma romdata CONFIG5L = 0x300008
#ifdef CODE_PROTECT
    const rom unsigned char config5L = 0b00000000;      // Code protect on
#else
    const rom unsigned char config5L = 0b11111111;      // Code protect off
#endif

    #pragma romdata CONFIG5H = 0x300009
#ifdef CODE_PROTECT
    const rom unsigned char config5H = 0b10000000;      // Bootblock on / EE protect off
#else
    const rom unsigned char config5H = 0b11111111;      // Bootblock off / EE protect off
#endif

    #pragma romdata CONFIG6L = 0x30000A
#ifdef Enable_Debug
    const rom unsigned char config6L = 0b11111111;      // No write protect if debugging
#else
    const rom unsigned char config6L = 0b11111000;      // Write protect on below 0xC000
#endif

    #pragma romdata CONFIG6H = 0x30000B
#ifdef Enable_Debug
    const rom unsigned char config6H = 0b11111111;      // No write protect if debugging
#else
    const rom unsigned char config6H = 0b10111111;      // Write protect on below 0xC000
#endif

    #pragma romdata CONFIG7L = 0x30000C
    const rom unsigned char config7L = 0b11111111;      // Read protect off

    #pragma romdata CONFIG7H = 0x30000D
    const rom unsigned char config7H = 0b11111111;      // Read protect off

    #pragma romdata


#else  // Toothpick Lite 
    
    #pragma romdata CONFIG1L = 0x01FFF8
#ifdef Enable_Debug 
    const rom unsigned char config1L = 0b00100000;     	// Debug, no extended instruction set, reset on stack under / over,  HW WD disabled
#else
    const rom unsigned char config1L = 0b10100000;     	// No Debug, no extended instruction set, reset on stack under / over,  HW WD disabled
#endif

    #pragma romdata CONFIG1H = 0x01FFF9
#ifdef CODE_PROTECT
    const rom unsigned char config1H = 0b11110000;      // Code protect, nop() on hi-nybble
#else
    const rom unsigned char config1H = 0b11110100;      // No Code protect, nop() on hi-nybble
#endif
 
    #pragma romdata CONFIG2L = 0x01FFFA
    const rom unsigned char config2L = 0b11000100;      // osc switchover enabled, failsafe clock mon enabled, HS oscillator

    #pragma romdata CONFIG2H = 0x01FFFB
    const rom unsigned char config2H = 0b11110111;      // 1:128 WDT prescaler, nop() on hi-nybble

    #pragma romdata CONFIG3L = 0x01FFFC
    const rom unsigned char config3L = 0b11111111;      // Microcontroller mode

    #pragma romdata CONFIG3H = 0x01FFFD
    const rom unsigned char config3H = 0b11111110;      // CCP2 mux RE7, nop() on hi-nybble

    #pragma romdata
#endif
