//------------------------------------------------------------------------------------------------------
//
//  HelloWorldBit.c
//
//  Hello World Bitstream tutorial
//
//  (c) FlexiPanel Ltd
//
//-------------------------------------------------------------------------------------------------------

#include "Toothpick.h" 
#include <p18cxxx.h>
 
rom unsigned char * szHello = "Hello\r\n";
rom unsigned char * szWorld = "World\r\n";

unsigned char LogicalChannel;

void main( void )
{
	overlay long lVal;
	overlay short Row, Col, sVal;
	overlay char cVal;

	// if no LinkMatik, flash red led rapidly
	while ((ToothpickSemaphores&TPSF_LMTEXISTS)==0)
	{
		LedRed = ~LedRed;
		msDelay(50);
	}

	// main loop
	while ( 1 ) 
	{
		// Flash LED to show we're alive
		LedGreen = ~LedGreen;
		msDelay( 250 );

	}  // end of while ( 1 ) 

}

// HighInterrupt handler
void HighInterrupt (void)
{
}

// LowInterrupt handler
void LowInterrupt (void)
{
	if (IsSWI( SWI_LMTData ) )
	{
		unsigned char ch = *LMTRxCh;

		// if the received character is an 'H', say hello
		if (ch=='h' || ch=='H')
		{
			LMTTransmit( szHello, 0, 7, 255, LogicalChannel );
		}

		// if the received character is an 'W', say world
		if (ch=='w' || ch=='W')
		{
			LMTTransmit( szWorld, 0, 7, 255, LogicalChannel );
		}

		// remove the character from the receive buffer
		LMTRxAdvanceCh;

		ClearSWI( SWI_LMTData );		// Clear interrupt flag
		return;
	}

	if (IsSWI( SWI_Tick ) )
	{
		ClearSWI( SWI_Tick );		// Clear interrupt flag
		return;
	}

}


// Error event handler 
void ErrorStatus( unsigned char ErrNum )
{
	// Flash error number for diagnositic purposes
	Breakpoint( ErrNum );
}


// BlueMatik Event handler 
void LMTEvent( unsigned char EventID, void *pData1, void *pData2 )
{
	if ( EventID==LMTE_Syntax )
	{
		// Flash error number for diagnositic purposes
		Breakpoint( ((unsigned char*)pData1)[0] );
	}

	if ( EventID==LMTE_Connected ) 
	{
		// turn on  red led during connection
		LedRed = LedRedOn;
		LogicalChannel = (unsigned char) pData2;
	}

	if ( EventID==LMTE_Disconnect ) 
	{
		// turn off red led after disconnection 
		LedRed = LedRedOff;
	}

}

// FxPEvent handler 
void FxPEvent( unsigned char EventID, void *pData )
{
}

