//---------------------------------------------------------------------------
//
//  HopCodes.h
//
//  Flexipanel public codes and structures - version 3.0
//
//---------------------------------------------------------------------------


// server flags sent to client 
#define MTF_Unicode						0x0001	// Data is unicode rather than multibyte (relevant on server greeting only)
#define MTF_NoPings						0x0002	// Device does not support Pings (sent on client / server greeting)
#define MTF_NoAcks						0x0004	// Device does not support Acks  (sent on client / server greeting)
#define MTF_AckPlease					0x0008	// Please acknowledge this message (MT_NewControlPanelFromServer, MT_ControlUpdateFromClient, MT_ControlUpdateFromServer only)
#define MTF_OneShot						0x0010	// Control panel is read only; it will disconnect after xmitting control panel; do not discard controls on server disconnect
#define MTF_ChecksumValid				0x0020	// Checksum member is valid 


// Possible control types
#define CtlTypeText			1
#define CtlTypeButton		2
#define CtlTypeLatch		3		
#define CtlTypeSection		4
#define CtlTypeDateTime		5
#define CtlTypeNumber		6
#define CtlTypeMatrix		7
#define CtlTypePassword		8
#define CtlTypeList			9
#define CtlTypeMessage		10
#define CtlTypeBlob			11
#define CtlTypeFile			12
#define CtlTypeUnknown		13  /* Control is not understood but best effort will be made to continue; add forward compatability */
#define CtlTypeImage		14

// message box states
#define MSG_INITIAL			1	// message box never shown
#define MSG_NONE			2	// message box managed to close without button being pressed (e.g. if server goes out of range, X pressed instead of button, etc)
#define MSG_AWAITING_REPLY	3	// message box believed to be in use

// flags which apply to all controls
#define CTL_INVISIBLE			0x00000001		// this control is not visible
#define CTL_RIGHTTOLEFT			0x00000002		// text runs right to left
#define CTL_UNICODE				0x00000004		// text is unicode (2 byte chars) 
#define CTL_STARTGROUP			0x00000008		// this control is the first of a series of controls which make sense to group together when displayed
#define CTL_ENSUREVISIBLE		0x00000010		// (during data update) ensure control is visible.  (what if inside section?)
#define CTL_ENDGROUP			0x00000020		// last of group of controls described by CTL_TXT_TITLEGROUP or CTL_STARTGROUP
#define CTL_COLCHANGED			0x00000040		// (during data update) control color is specified
#define CTL_DISABLED			0x00000080		// this control is not modifiable by the user

// Button Flags
#define CTL_BTT_RESET			0x00010000		// Perform system reset if pressed

// Text flags
#define CTL_TXT_MODIFIABLE		0x00010000		// use text is modifiable an edit box style
#define CTL_TXT_PASSWORD		0x00020000		// use password style  (applies to modifyable text only)

// Latch Flags
#define CTL_LCH_RADIORESET		0x00010000		// Clear all toggles in this radio group

// Section Flags
#define CTL_SCT_AUTOCLOSE		0x00010000		// closes if client disconnects

// Password Flags
#define CTL_PWD_MODIFIABLE		0x00010000		// use text is modifiable an edit box style
#define CTL_PWD_AUTOCLOSE		0x00020000		// closes if client disconnects

// Date/Time flags
#define CTL_DTM_MODIFIABLE		0x00010000		// date time is modifiable
#define CTL_DTM_MODIFYSECS		0x00020000		// seconds are modifiable
#define CTL_DTM_MODIFYMINS		0x00040000		// minutes are modifiable
#define CTL_DTM_MODIFYHOURS		0x00080000		// hours are modifiable
#define CTL_DTM_MODIFYWEEKDAY	0x00100000		// day of week is modifiable; behavior undefined if this and date/month/year is also modifiable, so don't specify both
#define CTL_DTM_MODIFYDAY		0x00200000		// date is modifiable
#define CTL_DTM_MODIFYMONTHS	0x00400000		// months are modifiable
#define CTL_DTM_MODIFYYEARS		0x00800000		// years are modifiable
#define CTL_DTM_MODIFYDATETIME	(CTL_DTM_MODIFIABLE|CTL_DTM_MODIFYSECS|CTL_DTM_MODIFYMINS|CTL_DTM_MODIFYHOURS|CTL_DTM_MODIFYDAY|CTL_DTM_MODIFYMONTHS|CTL_DTM_MODIFYYEARS)
#define CTL_DTM_MODIFYTIME		(CTL_DTM_MODIFIABLE|CTL_DTM_MODIFYMINS|CTL_DTM_MODIFYHOURS)
#define CTL_DTM_MODIFYWEEKTIME	(CTL_DTM_MODIFIABLE|CTL_DTM_MODIFYMINS|CTL_DTM_MODIFYHOURS|CTL_DTM_MODIFYWEEKDAY)
#define CTL_DTM_MODIFYDATE		(CTL_DTM_MODIFIABLE|CTL_DTM_MODIFYWEEKDAY|CTL_DTM_MODIFYMONTHS|CTL_DTM_MODIFYYEARS)
#define CTL_DTM_REALTIMECLOCK	0x01000000		// always reflects server's clock values
#define CTL_DTM_NOW				0x02000000		// initial value set to server clock value
#define freeclockoption			0x03000000		// since CTL_DTM_NOW and CTL_DTM_REALTIMECLOCK are mutually exclusive


// Number flags
#define CTL_NUM_MODIFIABLE		0x00010000		// number is modifiable
#define CTL_NUM_MIN				0x00020000		// Control has a maximum value
#define CTL_NUM_MAX				0x00040000		// Control has a minimum value
#define CTL_NUM_FORMAT_MASK		0x00300000		// Mask for extracting format value
#define CTL_NUM_FLOAT			0x00000000		// printf %.lf format
#define CTL_NUM_FIXEDPOINT		0x00100000		// printf %.nnlf format
#define CTL_NUM_EXPONENT		0x00200000		// printf %.nnle format
#define CTL_NUM_SHORTEST		0x00300000		// printf %.nnlg format
#define currentlyspare			0x00080000		// currently spare


// Matrix flags
#define CTL_MTX_DATA_MASK		0x00030000		// Data type mask
#define CTL_MTX_DATA_LIST		0x00000000		// No X data - y values are just a list of numbers of some sort
#define CTL_MTX_DATA_XY			0x00010000		// X Data are values
#define CTL_MTX_DATA_LABELS		0x00020000		// X Data are text labels
#define CTL_MTX_DATA_TY			0x00030000		// X data are DateTime values
#define CTL_MTX_X_BYTE_MASK		0x000C0000		// X data byte mask 
#define CTL_MTX_X_4BYTE			0x00000000		// X data are 4 byte signed 
#define CTL_MTX_X_2BYTE			0x00040000		// X data are 2 byte signed 
#define CTL_MTX_X_1BYTE			0x00080000		// X data are 1 byte signed 
#define CTL_MTX_Y_BYTE_MASK		0x00300000		// Y data byte mask 
#define CTL_MTX_Y_4BYTE			0x00000000		// Y data are 4 byte signed 
#define CTL_MTX_Y_2BYTE			0x00100000		// Y data are 2 byte signed 
#define CTL_MTX_Y_1BYTE			0x00200000		// Y data are 1 byte signed 
#define CTL_MTX_X_FORMAT_MASK	0x00C00000		// Mask for extracting format value
#define CTL_MTX_X_FLOAT			0x00000000		// printf %.lf format
#define CTL_MTX_X_FIXEDPOINT	0x00400000		// printf %.nnlf format
#define CTL_MTX_X_EXPONENT		0x00800000		// printf %.nnle format
#define CTL_MTX_X_SHORTEST		0x00C00000		// printf %.nnlg format
#define CTL_MTX_Y_FORMAT_MASK	0x03000000		// Mask for extracting format value
#define CTL_MTX_Y_FLOAT			0x00000000		// printf %.lf format
#define CTL_MTX_Y_FIXEDPOINT	0x01000000		// printf %.nnlf format
#define CTL_MTX_Y_EXPONENT		0x02000000		// printf %.nnle format
#define CTL_MTX_Y_SHORTEST		0x03000000		// printf %.nnlg format
#define CTL_MTX_ROW_UPDATE		0x04000000		// server updates use single ro update style 

// List Flags 
#define CTL_LST_NONULLSELECT	0x00010000		// client not permitted to select nothing

// Message flags 
#define CTL_MSG_ICON_MASK				0x00070000		
#define CTL_MSG_ICON_NONE				0x00010000		// Use no icon.  May be modified by FBVSSetControlProps
#define CTL_MSG_ICON_STOP				0x00020000		// Use Stop icon.  May be modified by FBVSSetControlProps
#define CTL_MSG_ICON_EXCLAMATION		0x00030000		// Use Exclamation icon.  May be modified by FBVSSetControlProps
#define CTL_MSG_ICON_QUESTION			0x00040000		// Use Question icon.  May be modified by FBVSSetControlProps
#define CTL_MSG_ICON_INFORMATION		0x00050000		// Use Information icon.  May be modified by FBVSSetControlProps
#define CTL_MSG_RESP_MASK				0x00700000		
#define CTL_MSG_RESP_NONE				0x00000000		// don't generate client reply 
#define CTL_MSG_RESP_OK					0x00100000		// display response button OK
#define CTL_MSG_RESP_OKCANCEL			0x00200000		// display response buttons OK, Cancel
#define CTL_MSG_RESP_RETRYCANCEL		0x00300000		// display response buttons Retry, Cancel
#define CTL_MSG_RESP_YESNO				0x00400000		// display response buttons Yes, No
#define CTL_MSG_RESP_YESNOCANCEL		0x00500000		// display response buttons Yes, No, Canecl
#define CTL_MSG_RESP_ABORTRETRYIGNORE	0x00600000		// display response buttons Abort, Retry, Ignore

// data flags
#define CTL_BLOB_TEXT				0x00010000		// data is text and must be converted multibyte / unicode where necessary
#define CTL_BLOB_HINT_MASK			0xFFF00000		
#define	CTL_BLOB_HINT_NONE			0x00000000		// data completely undefined for use with custom server / client combinations
#define	CTL_BLOB_HINT_URL			0x00100000		// data is a url

// file flags
#define CTL_FILE_HINT_MASK			0xFFF00000		
#define	CTL_FILE_HINT_NONE			0x00000000		// none - just store as files
#define	CTL_FILE_HINT_HTML			0x00100000		// first file is html

// image flags from V 3.0
#define CTL_IMG_FMT_MASK			0xFFF00000		
#define	CTL_IMG_FMT_BMP				0x00100000		// BMP file format - not supported in most 3.0 clients
#define	CTL_IMG_FMT_GIF				0x00200000		// GIF file format
#define	CTL_IMG_CLICK_MSG			0x00010000		// client update means user clicks on image rather than image data being updated
#define	CTL_IMG_MODIFIABLE			0x00020000		// the client may modify the image  - not supported in most 3.0 clients



// Structure used for communicating date / time in unmanaged code version
typedef struct tagDateTime		// 8 bytes
{
	char			sec;	// 0-59
	char			min;	// 0-59
	char			hour;	// 0-23
	char			date;	// 1st of month = 1
	char			dow;	// 0 = sunday, 6 = saturday, 7 = unknown
	char			month;  // 1 = jan
	unsigned short	year;	// 0-65535.  
} DateTimeU;


