//------------------------------------------------------------------------------------------------------
//
//  Toothpick.h
//
//  main Toothpick services header 
//
//  (c) FlexiPanel Ltd
//
//-------------------------------------------------------------------------------------------------------
 
// Useful basic types

typedef unsigned char BYTE;                 // 8-bit
typedef unsigned short int WORD;            // 16-bit
#ifndef _WINDOWS
typedef unsigned short long THREEBYTE;      // 24-bit
#endif
typedef unsigned long DWORD;                // 32-bit

typedef union _WORD_VAL
{
    BYTE v[2];
    WORD Val;
    struct
    {
        BYTE LSB;
        BYTE MSB;
    } byte;
} WORD_VAL;

#ifndef _WINDOWS
typedef union _THREEBYTE_VAL
{
    BYTE v[3];
    THREEBYTE Val;
    struct
    {
        BYTE LSB;
        BYTE MidB;
        BYTE MSB;
    } byte;
    struct
    {
        BYTE dummy;
        WORD Val;
    } hiword;
    struct
    {
        WORD Val;
        BYTE dummy;
   } loword;
} THREEBYTE_VAL;
#endif

//-------------------------------------------------------------------------------------------------------

// following ifndef ensures MPLAB-specific stuff not included in build of FlexiPanel Designer
#ifndef _WINDOWS

//-------------------------------------------------------------------------------------------------------

// time variables

// xtal frequency
//#define TP_40MHz 
#define TP_24MHz 
//#define TP_20MHz 

// boolean variable
#define Bool	unsigned char
#define bool	Bool
#define True	0xFF
#define False	0x00
#define true	True
#define false	False

// ADD 16-bit address variable
#define ADD unsigned short int

// date / time structure
typedef struct tagDateTime		// 8 bytes
{
	char			sec;	// 0-59
	char			min;	// 0-59
	char			hour;	// 0-23
	char			date;	// 1st of month = 1
	char			dow;	// 0 = sunday, 6 = saturday, 7 = unknown
	char			month;  // 1 = jan
	unsigned short	year;	// 0-65535.  
} DateTimeU;

// daylight savings times
#define DST_SUMMER_FLAG		0x20
#define DST_NONE			0
#define DST_AUSTRALIA		1
#define DST_BAHAMAS			2
#define DST_BRAZIL			3
#define DST_CANADA			4
#define DST_CHILE			5
#define DST_CUBA			6
#define DST_E_EUROPE		7   /* Former USSR countries mostly */
#define DST_EGYPT			8
#define DST_FALKLANDS		9
#define DST_GREENLAND		10
#define DST_IRAN			11
#define DST_IRAQ			12
#define DST_ISRAEL			13
#define DST_KIRGISTAN		14
#define DST_LEBANON			15
#define DST_MEXICO			16
#define DST_NAMIBIA			17
#define DST_NEWZEALAND		18
#define DST_PALESTINE		19
#define DST_PARAGUAY		20
#define DST_RUSSIA			21
#define DST_SYRIA			22
#define DST_TASMANIA		23
#define DST_TONGA			24
#define DST_USA				25
#define DST_W_EUROPE		26
#define DST_NONE_SUMMER				(DST_NONE | DST_SUMMER_FLAG)  // functions exactly like DST_NONE
#define DST_AUSTRALIA_SUMMER		(DST_AUSTRALIA | DST_SUMMER_FLAG)
#define DST_BAHAMAS_SUMMER			(DST_BAHAMAS | DST_SUMMER_FLAG)
#define DST_BRAZIL_SUMMER			(DST_BRAZIL | DST_SUMMER_FLAG)
#define DST_CANADA_SUMMER			(DST_CANADA | DST_SUMMER_FLAG)
#define DST_CHILE_SUMMER			(DST_CHILE | DST_SUMMER_FLAG)
#define DST_CUBA_SUMMER				(DST_CUBA | DST_SUMMER_FLAG)
#define DST_E_EUROPE_SUMMER			(DST_E_EUROPE | DST_SUMMER_FLAG)
#define DST_EGYPT_SUMMER			(DST_EGYPT | DST_SUMMER_FLAG)
#define DST_FALKLANDS_SUMMER		(DST_FALKLANDS | DST_SUMMER_FLAG)
#define DST_GREENLAND_SUMMER		(DST_GREENLAND | DST_SUMMER_FLAG)
#define DST_IRAN_SUMMER				(DST_IRAN | DST_SUMMER_FLAG)
#define DST_IRAQ_SUMMER				(DST_IRAQ | DST_SUMMER_FLAG)
#define DST_ISRAEL_SUMMER			(DST_ISRAEL | DST_SUMMER_FLAG)
#define DST_KIRGISTAN_SUMMER		(DST_KIRGISTAN | DST_SUMMER_FLAG)
#define DST_LEBANON_SUMMER			(DST_LEBANON | DST_SUMMER_FLAG)
#define DST_MEXICO_SUMMER			(DST_MEXICO | DST_SUMMER_FLAG)
#define DST_NAMIBIA_SUMMER			(DST_NAMIBIA | DST_SUMMER_FLAG)
#define DST_NEWZEALAND_SUMMER		(DST_NEWZEALAND | DST_SUMMER_FLAG)
#define DST_PALESTINE_SUMMER		(DST_PALESTINE | DST_SUMMER_FLAG)
#define DST_PARAGUAY_SUMMER			(DST_PARAGUAY | DST_SUMMER_FLAG)
#define DST_RUSSIA_SUMMER			(DST_RUSSIA | DST_SUMMER_FLAG)
#define DST_SYRIA_SUMMER			(DST_SYRIA | DST_SUMMER_FLAG)
#define DST_TASMANIA_SUMMER			(DST_TASMANIA | DST_SUMMER_FLAG)
#define DST_TONGA_SUMMER			(DST_TONGA | DST_SUMMER_FLAG)
#define DST_USA_SUMMER				(DST_USA | DST_SUMMER_FLAG)
#define DST_W_EUROPE_SUMMER			(DST_W_EUROPE | DST_SUMMER_FLAG)

// Day of week calculator
// this is Gregorian calendar which was adopted variously between 1582 and 1919
// use of this algorithm for dates before 1920 requires understanding of Gregorian / Julian calendars
// this routine is deliberately not commented and is copyright FlexiPanel Ltd and only for use with its products
// obtain our permission before using is elsewhere
// dow must be unsigned char - received day of week
// year must be unsigned short - will not be modified
// month must be unsigned char - will not be modified
// date must be unsigned char - will not be modified
#define CalcDayOfWeek( dow, year, month, date ) \
			{ \
				unsigned short tmpYear = year; \
				if (month < 3) tmpYear--; \
				tmpYear = tmpYear % 400; \
				switch (tmpYear / 100) \
				{ \
					case 0: dow = 2;  break;  \
					case 1: dow = 0;  break;  \
					case 2: dow = 5;  break;  \
					case 3: dow = 3;  break;  \
				} \
				tmpYear = tmpYear % 100; \
				dow = (dow + (tmpYear / 4) * 5) % 7; \
				dow = dow + tmpYear % 4; \
				switch (month) \
				{ \
					case 1:  \
					case 5:  dow += 5;  break; \
					case 2:  \
					case 6:  dow += 1;  break; \
					case 3:  \
					case 11: dow += 0;  break; \
					case 4:  \
					case 7:  dow += 3;  break; \
					case 8:  dow += 6;  break; \
					case 9:  \
					case 12: dow += 2;  break; \
					case 10: dow += 4; \
				} \
				dow = (dow + date)%7; \
			}


//-------------------------------------------------------------------------------------------------------

// function definitions

// "callback" functions which you must provide
void main( void );
void HighInterrupt( void );
void LowInterrupt( void );
void ErrorStatus( unsigned char ErrorNum );
void LMTEvent( unsigned char EventID, void *pData1, void *pData2);
void FxPEvent( unsigned char EventID, void *pData );

// Toothpick services
void msDelay( unsigned char ms );
void Breakpoint( unsigned char FlashVal );
void LMTCommand( unsigned char CommandID, void *pData, rom void *pDataR );
Bool LMTTransmit( unsigned rom char * pTxDataR, unsigned char * pTxData, unsigned short nBytes, unsigned char msTimeout, unsigned char LogicalChannel  );
Bool LMTReceive( unsigned char * pRxData, unsigned char nBytes, unsigned char msTimeout  );
void FxPCommand( unsigned char CommandID, unsigned short CtlID, void * pData );
void GetBytes( unsigned char Storage, ADD Addr, void * Data, ADD nData  );
void SetBytes( unsigned char Storage, ADD Addr, rom void * pRData, void * pData, ADD nData );

//-------------------------------------------------------------------------------------------------------

// developer-accessible data defined in Toothpick.c

#ifndef __Toothpick_c__
extern volatile DateTimeU 		RealTimeClock;				// Real Time Clock, seconds to years
extern volatile unsigned char ToothpickSemaphores;  
extern volatile unsigned char ToothpickSemaphores2;  
extern volatile unsigned char DSTEvent;		// Daylight Savings Time rule in effect
extern volatile unsigned char SWIflags;  	// event(s) which triggered SWI
extern volatile unsigned char NumConnections;				// Number of devices connected
extern volatile unsigned char LMTRxStart;	// Receive buffer startpoint
extern volatile unsigned char LMTRxEnd;		// Receive buffer endpoint
extern volatile unsigned char FxPChannel;	// Logocal channel of FlexiPanel remote client
extern volatile unsigned char pRemoteBTAddr[]; // bluetooth address of last remote device to connect
extern volatile unsigned char LMTRXSource[];
extern volatile unsigned char LMTRxBuff[];	// Receive buffer 
extern unsigned char usDelayCounter; 		// if routine CyclesDelay16plus16times() is not used, this can be used for any purpose
#else
void SetHighSpeedUART( void );
#endif

#define TPSF_LMTEXISTS			0x01		// Set if LinkMatik initialized OK.  If never initializes module may be damaged or missing module.  
#define TPSF_INATCOMMAND		0x02		// Set while AT command being processed but not complete
#define TPSF_SUSPENDTX			0x04		// Set while AT command is being sent 
#define TPSF_CONNECTING			0x08		// Set while connection being made but not complete
#define TPSF_CONNECTRESULT		0x10		// Set if connection completed OK
#define TPSF_FLEXIPANELSERVICE	0x20		// Set if FlexiPanel service is running
#define TPSF_FXPDEVICECONNECTED	0x40		// FlexiPanel device is connected
#define TPSF_FXPDATAINIT		0x80		// FlexiPanel data is initialized 
#define TPSF2_NONE_USED			0x01		
#define ClearSemaphores 	{ ToothpickSemaphores = 0; ToothpickSemaphores2 = 0; }
#define LinkMatikExists		((ToothpickSemaphores&TPSF_LMTEXISTS)!=0)
#define ProcessingCommand	((ToothpickSemaphores&TPSF_INATCOMMAND)!=0) 
#define ProcessingConnect	((ToothpickSemaphores&TPSF_CONNECTING)!=0) 
#define InFxPMode			((ToothpickSemaphores&TPSF_FLEXIPANELSERVICE)!=0) 
#define FxPDeviceConnected	((ToothpickSemaphores&TPSF_FXPDEVICECONNECTED)!=0) 
#define IsFxPInitialized	((ToothpickSemaphores&TPSF_FXPDATAINIT)!=0) 

// Toothpick settings
#define TPS_RealTimeClock			0x01	// implement real time clock	
#define TPS_ClientTick				0x02	// if a control is tied to RTC and client connected, retransmit time every second
#define TPS_HiIntOnTick				0x20	// generate a high interrupt on rtc tick instead of incrementing real time clock value	

#define SZPINMAX 17		/* do not change from 17 */
#define BTADDRLEN 6		/* do not modify from 6 */
#define VERSTRLEN 10	/* do not modify from 10 */
#define LOCALNAMELEN 32	/* do not modify from 32 - this value is max length including zero terminator*/
 
//-------------------------------------------------------------------------------------------------------

// error numbers

#define ERR_LMTATBUFFOVERRUN 	0x01		// AT response buffer overrun
#define ERR_LMTFRAME 			0x02		// Frame error on LinkMatik
#define ERR_LMTOVERRUN 			0x03		// Overrun error on LinkMatik
#define ERR_NOLINKMATIK			0x04 		// LinkMatik dependent operation attempted when LinkMatik not installed
#define ERR_WAITINGPREV			0x05		// Still waiting for previous AT command to complete
#define ERR_LMTBUFFOVERRUN		0x06		// LinkMatik incoming buffer overran (despite using CTS! - shouldn't happen)
#define ERR_FLEXIPANELSERVICE	0x07		// FlexiPanel Service is operating
#define ERR_NOFLEXIPANELSERVICE	0x08		// FlexiPanel Service is not operating
#define ERR_RXTIMEOUT			0x09		// LMTReceive timeout
#define ERR_TXTIMEOUT			0x0A		// LMTTransmit timeout
#define ERR_MEMORYFAILURE		0x0B		// Non-volatile memory failure
#define ERR_ARGUMENTINERROR		0x0C		// Argument to Toothpick function does not make sense
#define ERR_SYNCERROR			0x0D		// Linkmatik MUX mode sync error 
#define ERR_INITFAIL			0x0E		// Initialization failure
#define ERR_DOUBLETRANSMIT		0x0F		// An interrupt attempted to transmit to the LinkMatik something at the same time as another part of the code
#define ERR_BATCH_MISMATCH		0x10		// Batch transmit data did not match expected batch length

//-------------------------------------------------------------------------------------------------------

// Single bit digital I/O definitions

#define DirOutput		0
#define DirInput		1

#define DirAN0			TRISAbits.TRISA0
#define AN0Pin			PORTAbits.RA0
#define DirAN1			TRISAbits.TRISA1
#define AN1Pin			PORTAbits.RA1
#define DirAN2			TRISAbits.TRISA2
#define AN2Pin			PORTAbits.RA2
#define DirAN3			TRISAbits.TRISA3
#define AN3Pin			PORTAbits.RA3
#define DirAN4			TRISAbits.TRISA5
#define AN4Pin			PORTAbits.RA5
#define DirAN5			TRISFbits.TRISF0
#if defined( __18F67J10 )
	#define AN5Pin			TRISEbits.TRISE1
	#define DirAN6			PORTEbits.RE1
#else
	#define AN5Pin			PORTFbits.RF0
	#define DirAN6			TRISFbits.TRISF1
#endif
#define AN6Pin			PORTFbits.RF1
#define DirAN7			TRISFbits.TRISF2
#define AN7Pin			PORTFbits.RF2
#define DirAN8			TRISFbits.TRISF3
#define AN8Pin			PORTFbits.RF3
#define DirAN9			TRISFbits.TRISF4
#define AN9Pin			PORTFbits.RF4
#define DirAN10			TRISFbits.TRISF5
#define AN10Pin			PORTFbits.RF5
#define DirAN11			TRISFbits.TRISF6
#define AN11Pin			PORTFbits.RF6
#define DirCCP1			TRISCbits.TRISC2
#define CCP1Pin			PORTCbits.RC2
#define DirCCP2			TRISEbits.TRISE7
#define CCP2Pin			PORTEbits.RE7
#define DirCCP3			TRISGbits.TRISG0
#define CCP3Pin			PORTGbits.RG0
#define DirCCP4			TRISGbits.TRISG3
#define CCP4Pin			PORTGbits.RG3
#define DirCCP5			TRISGbits.TRISG4
#define CCP5Pin			PORTGbits.RG4
#define DirINT0			TRISBbits.TRISB7  /* Multiplexed with RB0; RB0 always an input */
#define INT0Pin			PORTBbits.RB7
#define DirINT1			TRISBbits.TRISB6  /* Multiplexed with RB1; RB1 always an input */
#define INT1Pin			PORTBbits.RB6
#define DirRxD			TRISGbits.TRISG2
#define RxDPin			PORTGbits.RG2
#define DirSCL			TRISCbits.TRISC3
#define SCLPin			PORTCbits.RC3
#define DirSDA			TRISCbits.TRISC4
#define SDAPin			PORTCbits.RC4
#define DirSDO			TRISCbits.TRISC5  /* Multiplexed with RA4 */
#define SDOPin			PORTCbits.RC5
#define DirSDO_A4		TRISAbits.TRISA4  /* RA4 alternate; one must always be an input */
#define SDOPin_A4		PORTAbits.RA4
#define DirTxD			TRISGbits.TRISG1
#define TxDPin			PORTGbits.RG1

//-------------------------------------------------------------------------------------------------------

// Parallel digital I/O definitions

#define DirParOutput		0x00
#define DirParInput			0xFF
#define DirPA7( dirval )	TRISF = (TRISF & 0x80) | (  (dirval)     & 0x7F);
#define DirPA6( dirval )	TRISF = (TRISF & 0x81) | ( ((dirval)<<1) & 0x7E);
#define DirPA5( dirval )	TRISF = (TRISF & 0x83) | ( ((dirval)<<2) & 0x7C);
#define DirPA4( dirval )	TRISF = (TRISF & 0x87) | ( ((dirval)<<3) & 0x78);
#define DirPA3( dirval )	TRISF = (TRISF & 0x8F) | ( ((dirval)<<4) & 0x70);
#define DirPA2( dirval )	TRISF = (TRISF & 0x9F) | ( ((dirval)<<5) & 0x60);
#define DirPB5( dirval )	TRISA = (TRISA & 0xD0) | (  (dirval)     & 0x2F);
#define DirPB4( dirval )	TRISA = (TRISA & 0xD1) | (  (dirval)     & 0x2E);
#define DirPB3( dirval )	TRISA = (TRISA & 0xD3) | ( ((dirval)<<1) & 0x2C);
#define DirPB2( dirval )	TRISA = (TRISA & 0xD7) | ( ((dirval)<<2) & 0x28);
#define DirPC5( dirval )	TRISG = (TRISG & 0xE0) | (  (dirval)     & 0x1F);
#define DirPC4( dirval )	TRISG = (TRISG & 0xE1) | ( ((dirval)<<1) & 0x1E);
#define DirPC3( dirval )	TRISG = (TRISG & 0xE3) | ( ((dirval)<<2) & 0x1C);
#define DirPC2( dirval )	TRISG = (TRISG & 0xE7) | ( ((dirval)<<3) & 0x18);
#define SetPA7( val )		PORTF = (PORTF & 0x80) | (  (val)     & 0x7F);
#define SetPA6( val )		PORTF = (PORTF & 0x81) | ( ((val)<<1) & 0x7E);
#define SetPA5( val )		PORTF = (PORTF & 0x83) | ( ((val)<<2) & 0x7C);
#define SetPA4( val )		PORTF = (PORTF & 0x87) | ( ((val)<<3) & 0x78);
#define SetPA3( val )		PORTF = (PORTF & 0x8F) | ( ((val)<<4) & 0x70);
#define SetPA2( val )		PORTF = (PORTF & 0x9F) | ( ((val)<<5) & 0x60);
#define SetPB5( val )		PORTA = (PORTA & 0xD0) | (  (val)     & 0x0F) | (  ((val)<<1) & 0x20);
#define SetPB4( val )		PORTA = (PORTA & 0xD0) | (  (val<<1)  & 0x0E) | (  ((val)<<2) & 0x20);
#define SetPB3( val )		PORTA = (PORTA & 0xD0) | (  (val<<2)  & 0x0C) | (  ((val)<<3) & 0x20);
#define SetPB2( val )		PORTA = (PORTA & 0xD0) | (  (val<<3)  & 0x08) | (  ((val)<<4) & 0x20);
#define SetPC5( val )		PORTG = (PORTG & 0xE0) | (  (val)     & 0x1F);
#define SetPC4( val )		PORTG = (PORTG & 0xE1) | ( ((val)<<1) & 0x1E);
#define SetPC3( val )		PORTG = (PORTG & 0xE3) | ( ((val)<<2) & 0x1C);
#define SetPC2( val )		PORTG = (PORTG & 0xE7) | ( ((val)<<3) & 0x18);
#define GetPA7				 (PORTF & 0x7F)
#define GetPA6				((PORTF & 0x7E)>>1)
#define GetPA5				((PORTF & 0x7C)>>2)
#define GetPA4				((PORTF & 0x78)>>3)
#define GetPA3				((PORTF & 0x70)>>4)
#define GetPA2				((PORTF & 0x60)>>5)
#define GetPB5				((PORTA & 0x0F) | ((PORTA>>1) & 0x10))
#define GetPB4				(((PORTA & 0x0F) | ((PORTA>>1) & 0x10) >> 1))
#define GetPB3				(((PORTA & 0x0F) | ((PORTA>>1) & 0x10) >> 2))
#define GetPB2				(((PORTA & 0x0F) | ((PORTA>>1) & 0x10) >> 3))
#define GetPC5				 (PORTG & 0x1F)
#define GetPC4				((PORTG & 0x1E)>>1)
#define GetPC3				((PORTG & 0x1C)>>2)
#define GetPC2				((PORTG & 0x18)>>3)

//-------------------------------------------------------------------------------------------------------

// A to D definitions

#define ADConverterOn10bit	{ \
								ADCON0 |= 0x01; \
								ADCON2 = 0b10000110;  /* a/d conv period 64Tosc, right shift result */ \
							}

#define ADConverterOn8bit	{ \
								ADCON0 |= 0x01; \
								ADCON2 = 0b00000110;  /* a/d conv period 64Tosc, left shift result */ \
							}
#define ADConverterOff 		ADCON0 &= 0xFE; 

#define VRefNegIsVss		ADCON1bits.VCFG1 = 0;
#define VRefNegIsAN2		ADCON1bits.VCFG1 = 1;	// AN2 should be analog
#define VRefPosIsVdd		ADCON1bits.VCFG0 = 0;
#define VRefPosIsAN3		ADCON1bits.VCFG0 = 1;  	// AN3 should be analog

#define SetAnalogNone		ADCON1 = (ADCON1 & 0xF0) | 0x0F;
#define SetAnalogAD0		ADCON1 = (ADCON1 & 0xF0) | 0x0E;
#define SetAnalogAD0toAD1	ADCON1 = (ADCON1 & 0xF0) | 0x0D;
#define SetAnalogAD0toAD2	ADCON1 = (ADCON1 & 0xF0) | 0x0C;
#define SetAnalogAD0toAD3	ADCON1 = (ADCON1 & 0xF0) | 0x0B;
#define SetAnalogAD0toAD4	ADCON1 = (ADCON1 & 0xF0) | 0x0A;
#define SetAnalogAD0toAD5	ADCON1 = (ADCON1 & 0xF0) | 0x09;
#define SetAnalogAD0toAD6	ADCON1 = (ADCON1 & 0xF0) | 0x08;
#define SetAnalogAD0toAD7	ADCON1 = (ADCON1 & 0xF0) | 0x07;
#define SetAnalogAD0toAD8	ADCON1 = (ADCON1 & 0xF0) | 0x06;
#define SetAnalogAD0toAD9	ADCON1 = (ADCON1 & 0xF0) | 0x05;
#define SetAnalogAD0toAD10	ADCON1 = (ADCON1 & 0xF0) | 0x04;
#define SetAnalogAD0toAD11	ADCON1 = (ADCON1 & 0xF0) | 0x03;

#define SetADChan( ch ) 	ADCON0 = (((unsigned char)ch)<<2) + (ADCON0 & 0x03); 
#define StartAtoD			{ ADRES = 0; ADCON0bits.GO = 1; }
#define AtoDInProgress		(ADCON0bits.NOT_DONE) 
#define AwaitAtoDComplete	{ while (ADCON0bits.NOT_DONE) {;}; }

#define GetADResult8bit( uData )	{ uData = ADRESH; }
#define GetADResult10bit( uData )	{ uData = ADRESL;   ((unsigned char *)&uData)[1] = ADRESH; }

//-------------------------------------------------------------------------------------------------------

// delay routines

#define CyclesDelay3p2plus3p2times( usDel )		{ for (usDelayCounter=usDel; usDelayCounter>0; usDelayCounter--) {;} }  

//-------------------------------------------------------------------------------------------------------

// LinkMatik definitions

#define LMTRxD			PORTCbits.RC6  			/*  Output from PIC TX pin to LinkMatik */ 
#define	DirLMTRxD		TRISCbits.TRISC6
#define LMTRxDInit		1
#define LMTTxD			PORTCbits.RC7  			/*  Input to PIC RX pin from LinkMatik */ 
#define	DirLMTTxD		TRISCbits.TRISC7

#define LMTCTS			PORTDbits.RD6  			/*  Flow control output from PIC to LinkMatik, logic 0 for clear to send, logic 1 for stop sending */ 
#define	DirLMTCTS		TRISDbits.TRISD6		
#define LMTRTS			PORTBbits.RB2  			/*  Flow control input to PIC from LinkMatik, logic 0 for clear to send, logic 1 for stop sending */ 
#define	DirLMTRTS		TRISBbits.TRISB2		

#define LMTRstPin		PORTEbits.RE3  			/*  Pulse high for 10ms to reset LinkMatik */ 
#define	DirLMTRstPin	TRISEbits.TRISE3

#define LMTTxTrans		PORTEbits.RE4
#define	DirLMTTxTrans	TRISEbits.TRISE4

#define LMTRstPulse		1
#define LMTRstRelease	0
#define LMTCTSStop		1			  			
#define LMTCTSClear		0			  			
#define LMTRTSStop		1			  			
#define LMTRTSClear		0

#ifdef TP_40MHz
#define LMTBaudBrghOne9600		255			/* 1.70% fast, (9.766 kbaud) */
#define LMTBaudBrghOne19200		129			/* 0.16% fast, (19.231 kbaud) */
#define LMTBaudBrghOne38400		64			/* 0.16% fast, (38.461 kbaud) */
#define LMTBaudBrghOne57600		42			/* 0.93% fast, (58.139 kbaud) */
#define LMTBaudBrghOne115200	21			/* 1.38% slow, (113.64 kbaud) */
#elif defined( TP_20MHz )
#define LMTBaudBrghOne9600		129			/* 1.70% fast, (9.766 kbaud) */
#define LMTBaudBrghOne19200		64			/* 0.16% fast, (19.231 kbaud) */
#define LMTBaudBrghOne38400		32			/* 0.16% fast, (38.461 kbaud) */
#define LMTBaudBrghOne57600		21			/* 0.93% fast, (58.139 kbaud) */
#define LMTBaudBrghOne115200	10			/* 1.38% slow, (113.64 kbaud) */
#elif defined( TP_24MHz )
#define LMTBaudBrghOne9600		155			/* 0.16% fast, (9.615 kbaud) */
#define LMTBaudBrghOne19200		77			/* 0.16% fast, (19.231 kbaud) */
#define LMTBaudBrghOne38400		38			/* 0.16% fast, (38.462 kbaud) */
#define LMTBaudBrghOne57600		25			/* 0.16% fast, (57.692 kbaud) */
#define LMTBaudBrghOne115200	12			/* 0.16% fast, (115.35 kbaud) */
#else
  #error "TP_xxMHz not defined" 
#endif


#define AwaitLMTComplete()	while (ProcessingCommand) {;};

#define LMTUARTFrameErr		RCSTA1bits.FERR
#define LMTUARTOverrunErr	RCSTA1bits.OERR
#define LMTUARTEnable		RCSTA1bits.CREN
#define LMTUARTBaud			SPBRG1

///--------------------------------------------------------------------------------------------

// LinkMatik Events IDs
#define LMTE_OK				0x00	/* last command completed OK */
#define LMTE_Syntax			0x02	/* warning message */
#define LMTE_Connected		0x03	/* Serial connection established */
#define LMTE_Disconnect		0x04	/* Serial connection terminated */
#define LMTE_Found			0x05	/* Remote device found during inquiry */
#define LMTE_Paired			0x06	/* Device successfully paired */
#define LMTE_BER			0x07	/* Bit error rate result */
#define LMTE_RSSI			0x08	/* Signal strength result */
#define LMTE_HostProgram	0x09	/* Host program - used for programming Stamps in Toothpick Stamp Edition */
#define LMTE_Generic		0xFF	/* Other response, e.g. from general LinkMatik command */

//--------------------------------------------------------------------------------------------

// LinkMatik Command IDs

#define LMTC_Inquiry	0x01	/* Enter inquiry mode */
#define LMTC_Connect	0x02	/* Connect */
#define LMTC_Disconnect	0x03	/* Disconnect */
#define LMTC_SetLink	0x04	/* Set link state */
#define LMTC_Sleep		0x05	/* Go into low power state */
#define LMTC_BER		0x06	/* Enquire bit error rate */
#define LMTC_RSSI		0x07	/* Enquire signal strength */
#define LMTC_Generic	0xFF	/* General command */

//--------------------------------------------------------------------------------------------

// Bluetooth Device Class constants  

// these binary formats are not supported by LMT 2.0; the value must be specified in ASCII
// however, we've left them in for your reference

// To get your device class, bitwise-OR together:
// (1) As many Svc_ service values as apply 
// (2) One Dev_ device class value as appropriate
// (3) if the Dev_ was one of the Dev_Pr_ values, then also one PrDev_ value
// (4) if the Dev_ was Dev_Imaging, then also as many ImDev_ values as apply

//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	Svc_LtdDisc			0b000000000010000000000000
#define	Svc_Position		0b000000010000000000000000
#define	Svc_Network			0b000000100000000000000000
#define	Svc_Render			0b000001000000000000000000
#define	Svc_Capture			0b000010000000000000000000
#define	Svc_Object			0b000100000000000000000000
#define	Svc_Audio			0b001000000000000000000000
#define	Svc_Phone			0b010000000000000000000000
#define	Svc_Info			0b100000000000000000000000
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	Dev_Cmp_Desktop		0b000000000000000100000100
#define	Dev_Cmp_Server		0b000000000000000100001000
#define	Dev_Cmp_Laptop		0b000000000000000100001100
#define	Dev_Cmp_Handheld	0b000000000000000100010000
#define	Dev_Cmp_Palm		0b000000000000000100010100
#define	Dev_Cmp_Wearable	0b000000000000000100011000
#define	Dev_Cmp_Other		0b000000000000000100000000
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	Dev_Ph_Cell			0b000000000000001000000100
#define	Dev_Ph_Cordless		0b000000000000001000001000
#define	Dev_Ph_Smart		0b000000000000001000001100
#define	Dev_Ph_Gateway		0b000000000000001000010000
#define	Dev_Ph_ISDN			0b000000000000001000010100
#define	Dev_Ph_Other		0b000000000000001000000000
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	Dev_LAN_0			0b000000000000001100000000
#define	Dev_LAN_1_17		0b000000000000001100100000
#define	Dev_LAN_17_33		0b000000000000001101000000
#define	Dev_LAN_33_50		0b000000000000001101100000
#define	Dev_LAN_50_67		0b000000000000001110000000
#define	Dev_LAN_67_83		0b000000000000001110100000
#define	Dev_LAN_83_99		0b000000000000001111000000
#define	Dev_LAN_100			0b000000000000001111100000
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	Dev_AV_WearHead		0b000000000000010000000100
#define	Dev_AV_HandsFree	0b000000000000010000001000
#define	Dev_AV_Micro		0b000000000000010000010000
#define	Dev_AV_Loudspk		0b000000000000010000010100
#define	Dev_AV_Head			0b000000000000010000011000
#define	Dev_AV_Walkman		0b000000000000010000011100
#define	Dev_AV_CarRadio		0b000000000000010000100000
#define	Dev_AV_SetTop		0b000000000000010000100100
#define	Dev_AV_HiFi			0b000000000000010000101000
#define	Dev_AV_DVD			0b000000000000010000101100
#define	Dev_AV_VidCam		0b000000000000010000110000
#define	Dev_AV_CamCord		0b000000000000010000110100
#define	Dev_AV_Monitor		0b000000000000010000111000
#define	Dev_AV_TV			0b000000000000010000111100
#define	Dev_AV_Confrnce		0b000000000000010001000000
#define	Dev_AV_Toy			0b000000000000010001001000
#define	Dev_AV_Other		0b000000000000010000000000
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	Dev_Pr_NKNP			0b000000000000010100000000  /* No keyboard no pointing device */
#define	Dev_Pr_K			0b000000000000010101000000  /* keyboard but no pointing device */
#define	Dev_Pr_P			0b000000000000010110000000  /* pointing device but no keyboard */
#define	Dev_Pr_KP			0b000000000000010111000000  /* keyboard and pointing device */
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	PrDev_Joystick		0b000000000000000000000100
#define	PrDev_Gamepad		0b000000000000000000001000
#define	PrDev_Remote		0b000000000000000000001100
#define	PrDev_Sensor		0b000000000000000000010000
#define	PrDev_Digitizer		0b000000000000000000010100
#define	PrDev_Card			0b000000000000000000011000
#define	PrDev_Other			0b000000000000000000000000
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	Dev_Imaging			0b000000000000011000000000
#define	ImDev_Display		0b000000000000000000010000
#define	ImDev_Camera		0b000000000000000000100000
#define	ImDev_Scanner		0b000000000000000001000000
#define	ImDev_Printer		0b000000000000000010000000
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00
#define	Dev_Misc			0b000000000000000000000000
#define	Dev_NoCat			0b000000000001111100000000
//-----nybble-marker----------842184218421842184218421
//-----bitfield--marker-------|--Srvc---||Maj||Mnr-|00

#define	Search_GIAC			0x9E8B33	/* Use this code in a general inquiry for devices */
#define	Search_LIAC			0x9E8B00	/* Use this code for limited-period inquiry by user request - see Generic Access Profile, Bluetooth SIG */

//--------------------------------------------------------------------------------------------

// LinkMatik Buffered I/O macros; these assume buffer length of 256 byte and use
// arithmetic overflow to achieve buffer circularity

#define LMTRxIntFlag		PIR1bits.RCIF
#define LMTChIn				RCREG1
#define LMTChOut			TXREG1
#define LMTNotTxmitting		TXSTA1bits.TRMT

#define LMT_CLEAR_THRESHOLD	250
#define LMT_SET_THRESHOLD	250
#define LMTRxNChar 			( (unsigned char) (LMTRxEnd - LMTRxStart ) )
#define LMTRxSpace			( (unsigned char) ( 0xFF - LMTRxNChar ) )
#define LMTRxEmpty 			( LMTRxEnd == LMTRxStart )
#define LMTRxAdvance( nch )	{ \
                            	LMTRxStart += nch; \
								if ( LMTRxNChar < LMT_CLEAR_THRESHOLD )  LMTCTS = LMTCTSClear; \
							}

#define LMTRxAdvanceCh		{ \
                            	LMTRxStart ++; \
								if ( LMTRxNChar < LMT_CLEAR_THRESHOLD )  LMTCTS = LMTCTSClear; \
							}

#define LMTRxLoc( i )		( LMTRxBuff + ( LMTRxStart + ( (unsigned char) i ) ) )
#define LMTRxCh				( LMTRxBuff + LMTRxStart )

#define SetLMTRxSource( j, LogicalChan ) \
							{\
								char * pByte = LMTRXSource + ( (unsigned char) ((j)>>2));\
								*pByte = *pByte & (( 0x03 << (((j)&0x03)<<1) ) ^ 0xFF)  | ((LogicalChan) << (((j)&0x03)<<1));\
							}
#define LMTRxSource( i )	(( *( LMTRXSource + ( (unsigned char) ((LMTRxStart+(i))>>2) ) ) >> (((LMTRxStart+(i))&0x03)<<1) ) & 0x03)
#define LMTRxChSource		(( *( LMTRXSource + ( (unsigned char) (LMTRxStart>>2) ) ) >> (((LMTRxStart)&0x03)<<1) ) & 0x03)
											
#define ResetLMTRx			{										\
								DisableHighInts;					\
								LMTRxEnd = 0;						\
								LMTRxStart = 0;						\
								EnableHighInts;						\
							}

#define LMTRxWaitBytes(i)	while ( ((unsigned char) i) > LMTRxNChar )  {;}



//--------------------------------------------------------------------------------------------

// Timer 2 as low priority interrupt invoked from software

// initialize timer 2
#define SWIInit 			{  \
								PIE1bits.TMR2IE = 0;  /* interrupt disabled   */ \
								IPR1bits.TMR2IP = 0;  /* interrupt low priority  */ \
								T2CON = 0x04; /* on, not pre- or post-scale */ \
								PR2	= 0x01; /* timer generated interrupt every clock cycle */ \
							}

#define SetSWI( IntFlag )	{  \
								SWIflags |= (IntFlag);  \
								PIE1bits.TMR2IE = 1;  \
							}

#define ClearSWI( IntFlag )	{  \
								SWIflags &= ~(IntFlag);  \
								if (SWIflags == 0)  \
								{  \
									PIE1bits.TMR2IE = 0;  \
									PIR1bits.TMR2IF = 0;  \
								}  \
							}
#define IsSWI( IntFlag )	( PIR1bits.TMR2IF && (SWIflags & IntFlag )!=0)


#define SWI_ATResponse		0x01
#define SWI_FxPData			0x02
#define SWI_LMTData			0x04
#define SWI_RefreshDialog	0x08
#define SWI_Tick			0x10
#define SWI_SWI3			0x20  // Free for developer use
#define SWI_SWI2			0x40  // Free for developer use
#define SWI_SWI1			0x80  // Free for developer use


//--------------------------------------------------------------------------------------------

// led, Pushbutton definitions
#define LedGreen 		PORTEbits.RE5
#define	DirLedGreen		TRISEbits.TRISE5
#define	LedGreenOn		1
#define LedGreenOff		0
#define LedRed	 		PORTDbits.RD4
#define	DirLedRed		TRISDbits.TRISD4
#define	LedRedOn		1
#define LedRedOff		0
#define PushButton	 	PORTBbits.RB3
#define	DirPushButton	TRISBbits.TRISB3
#define	PushButtonOn	0  						/* Active low */
#define PushButtonOff	1

#define InitPBandLEDs	{								\
							LedGreen = LedGreenOff;  	\
							DirLedGreen = DirOutput;  	\
							LedRed = LedRedOff;			\
							DirLedRed = DirOutput;		\
							DirPushButton = DirInput; 	\
						}

#define EnablePBIntHighDown		{											\
									INTCON2bits.INTEDG3 = PushButtonOn; 	\
									INTCON2bits.INT3P = 1;					\
									INTCON3bits.INT3IE = 1;					\
								}

#define EnablePBIntHighUp		{											\
									INTCON2bits.INTEDG3 = PushButtonOff; 	\
									INTCON2bits.INT3P = 1;					\
									INTCON3bits.INT3IE = 1;					\
								}

#define EnablePBIntLowDown		{											\
									INTCON2bits.INTEDG3 = PushButtonOn; 	\
									INTCON2bits.INT3P = 0;					\
									INTCON3bits.INT3IE = 1;					\
								}

#define EnablePBIntLowUp		{											\
									INTCON2bits.INTEDG3 = PushButtonOff; 	\
									INTCON2bits.INT3P = 0;					\
									INTCON3bits.INT3IE = 1;					\
								}

#define DisablePBInt			INTCON3bits.INT3IE = 0;
#define PushbuttonIntFlag		INTCON3bits.INT3IF

//--------------------------------------------------------------------------------------------

// Interrupts (high / low prioritized interrupts model used) 

// caution, do not EnableLowInts inside an low interrupt unless you really mean it.  It can cause a nested interrupt to start.
// So for example, do this:
//
//  unsigned char LIS = LowIntState;
//  DisableLowInts;
//  --- your critical code ---
//  if (LIS) EnableLowInts;
//
// rather than this
//
//  DisableLowInts;
//  --- your critical code ---
//  EnableLowInts;

#define SetPrioritizedInts		RCONbits.IPEN = 1;		
#define EnableHighInts			INTCONbits.GIEH = 1;
#define EnableLowInts			INTCONbits.GIEL = 1;
#define DisableHighInts			INTCONbits.GIEH = 0;
#define DisableLowInts			INTCONbits.GIEL = 0;
#define LowIntState				INTCONbits.GIEL

// all possible interrupt flags

#define IntRB0 			INTCONbits.INT0IF	
#define IntRB1			INTCON3bits.INT1IF
#define IntRB2			INTCON3bits.INT2IF
#define IntRB3 			INTCON3bits.INT3IF
#define IntRB4toRB7		INTCONbits.RBIF 
#define IntTimer0		INTCONbits.TMR0IF
#define IntTimer1		PIR1bits.TMR1IF
#define IntTimer2		PIR1bits.TMR2IF
#define IntTimer3		PIR2bits.TMR3IF
#define IntTimer4		PIR3bits.TMR4IF
#define IntCCP1			PIR1bits.CCP1IF
#define IntCCP2			PIR2bits.CCP2IF
#define IntCCP3			PIR3bits.CCP3IF
#define IntCCP4			PIR3bits.CCP4IF
#define IntCCP5			PIR3bits.CCP5IF
#define IntPSP			PIR1bits.PSPIF
#define IntA2D			PIR1bits.ADIF 
#define IntSSP			PIR1bits.SSPIF 
#define IntComparator	PIR2bits.CMIF 
#define IntEE			PIR2bits.EEIF 
#define IntBusCrash		PIR2bits.BCLIF
#define IntLowVolt		PIR2bits.LVDIF
#define IntRxD1			PIR1bits.RC1IF
#define IntTxD1			PIR1bits.TX1IF
#define IntRxD2			PIR3bits.RC2IF 
#define IntTxD2			PIR3bits.TX2IF 

///--------------------------------------------------------------------------------------------

// Reset / wakeup flags describe what caused the last reset / wakeup 

// These defines only valid if InitResetWakupFlags was called prior to reset or wakeup event

#define ResetOrWakeEventWasStackOverflow				(STKPTRbits.STKFUL!=0 && (RCON & 0b00000011) == 0b00000011)
#define ResetOrWakeEventWasStackUnderflow				(STKPTRbits.STKUNF!=0 && (RCON & 0b00000011) == 0b00000011)
#define ResetOrWakeEventWasSoftwareReset				(((RCON & 0b00010000) == 0b00000000) && STKPTRbits.STKFUL==0 && STKPTRbits.STKUNF==0)
#define ResetOrWakeEventWasInterrtuptOrNMCLRDuringSleep	(((RCON & 0b00001100) == 0b00001000) && STKPTRbits.STKFUL==0 && STKPTRbits.STKUNF==0)
#define ResetOrWakeEventWasWDTReset						(((RCON & 0b00001100) == 0b00000100) && STKPTRbits.STKFUL==0 && STKPTRbits.STKUNF==0)
#define ResetOrWakeEventWasWDTWakeup					(((RCON & 0b00001100) == 0b00000000) && STKPTRbits.STKFUL==0 && STKPTRbits.STKUNF==0)
#define ResetOrWakeEventWasBrownOut						(((RCON & 0b00011101) == 0b00011100) && STKPTRbits.STKFUL==0 && STKPTRbits.STKUNF==0)
#define InitResetWakupFlags 							{  STKPTR &= 0x3F;  RCON |= 0x1F;  }

//--------------------------------------------------------------------------------------------

// Real time clock (RTC) & oscillator definitions
// real time clock variable RealTimeClock is defined in Toothpick.c

#define Osc32kHzT1Init		T1CON = 0x8B;	/* Turns on 32kHz osc and timer 1 on with RD16 set */	
#define Osc32kHzInit		{															\
								T1CONbits.T1OSCEN = 1; /* Turns on 32kHz osc only */	\
								T1CONbits.T1SYNC = 0;									\
							}

#define Osc32kHzWake		Osc32kHzT1Init; 
#define Osc32kHzSleep		T1CON = 0x00; 
#if (defined( __18F67J10 ) || defined( __18F6722 ))
#define SetSpeed6MIPs	 	OSCCONbits.SCS1 = 1; OSCCONbits.SCS0 = 0; 
#define SetSpeed8192IPs	 	OSCCONbits.SCS1 = 0; OSCCONbits.SCS0 = 1; 
  #define Is8192IPs			(OSCCONbits.SCS0)
#else
  #define Is8192IPs			(OSCCONbits.SCS)
#define SetSpeed6MIPs	 	OSCCONbits.SCS = 0; 
#define SetSpeed8192IPs	 	OSCCONbits.SCS = 1;
#endif
#define EnableT1LowInt		{ IPR1bits.TMR1IP = 0; PIE1bits.TMR1IE = 1; }
#define EnableT1HighInt		{ IPR1bits.TMR1IP = 1; PIE1bits.TMR1IE = 1; }
#define DisableT1Int		PIE1bits.TMR1IE = 0;
#define ClearSecFracs		{ TMR1H = 0x80; TMR1L = 0; }
#define SetT1Bit15			{ T1CONbits.RD16 = 0; TMR1H = 0x80; T1CONbits.RD16 = 1;  }
#define SetSecFracs( psF )	{ TMR1H = 0x80 | ((unsigned char*)psF)[1];  TMR1L = ((unsigned char*)psF)[0]; }
#define GetSecFracs( psF )	{ ((unsigned char*)psF)[0] = TMR1L; ((unsigned char*)psF)[1] = TMR1H & 0x7F; }
#define RTCIntFlag			PIR1bits.TMR1IF

///--------------------------------------------------------------------------------------------

// FlexiPanel user interface Event IDs

#define FxPE_Connect	0x01	/* device connected */
#define FxPE_Disco		0x02	/* device disconnected */
#define FxPE_Ack		0x03	/* message acknowledged */
#define FxPE_ClntUpdate	0x04	/* client updated value */
#define FxPE_PingReply	0x05	/* ping reply received */
#define FxPE_PingFail	0x06	/* ping failed */
#define FxPE_MsgResp	0x07	/* message box response received */
#define FxPE_NewDialog	0x08	/* dialog information has just been sent */
#define FxPE_Custom		0xFF	/* custom message received*/

//--------------------------------------------------------------------------------------------

// FlexiPanel user interface Command IDs

#define FxPC_Start					0x01	/* Start flexipanel service */
#define FxPC_Finish					0x02	/* End flexipanel service */
#define FxPC_SetDialog				0x03	/* Set dialog to be displayed */
#define FxPC_Disconnect				0x04	/* Disconnect from client */
#define FxPC_CtlUpdate				0x05	/* Update control value on client */
#define FxPC_PartUpdate				0x06	/* Update row value on client */
#define FxPC_SetProps				0x07	/* Update control properties value on client */
#define FxPC_SetColor				0x08	/* Update control color on client */
#define FxPC_InitData   			0x09	/* (Re-)Initialize all control data */ 
#define FxPC_MultiUpdate			0x0A	/* send multiple control updates */ 
#define FxPC_MultiPartUpdate		0x0B	/* send multiple partial control updates */ 
#define FxPC_MultiPropsUpdate		0x0C	/* send multiple control props updates */ 

// endif on ifdef MPLAB; anything hereafer will be included in FlexiPanel Designer compilation
#endif 

//--------------------------------------------------------------------------------------------

// FlexiPanel user interface data structure

// data comprises one bqFxPData structure only
// then one bqFxPDlgData structure per dialog
// then one bqFxPCtlData structure per control 
// then by variable length data linked by pointers from bqFxPCtlData fields
// then profile data (all profile integers are stored as two bytes and assume upper two bytes of each 4-byte integer are zero)
// all ADD addresses are TWO BYTE unsigned addresses relative to start of bqFxPData  

// message header
#ifndef SHORTHOPHEADER
#define SHORTHOPHEADER
#define HOPNAMELEN 8
typedef struct tagShortHopHeader
{
	char			Name[HOPNAMELEN];
	unsigned short	wSerial;			
	unsigned short	wCheckSum;			
	unsigned short	Version;
	unsigned short	LowestVersion;
} ShortHopHeader;
#endif

// memory storage types; STR_ROM treated same as STR_ROM01 and kept for backward compatibility
#define STR_ROM		0x01  
#define STR_RAM		0x02
#define STR_EE		0x03
#define STR_EXT0	0x10
#define STR_EXT1	0x11
#define STR_EXT2	0x12
#define STR_EXT3	0x13
#define STR_EXT4	0x14
#define STR_EXT5	0x15
#define STR_EXT6	0x16
#define STR_EXT7	0x17
#define STR_EXTi( iExt ) (0x10 + iExt )
#define STR_ROM00	0x80
#define STR_ROM01	0x81
#define STR_ROMi( iRom ) (0x80 + iRom )

// External memory initialization
#define I2CMemInit100kHz {\
		SSPADD = 0x3F;		 \
	  	SSPSTAT &= 0x3F;     \
	  	SSPCON1 = 0x00;      \
	  	SSPCON2 = 0x00;      \
	  	SSPCON1 |= 8;		 \
	  	SSPSTAT |= 0xC0;     \
		DDRCbits.RC3 = 1;    \
	  	DDRCbits.RC4 = 1;    \
		SSPCON1 |= 0x20;  }   

#define I2CMemInit400kHz {\
		SSPADD = 0x3F;		 \
	  	SSPSTAT &= 0x3F;     \
	  	SSPCON1 = 0x00;      \
	  	SSPCON2 = 0x00;      \
	  	SSPCON1 |= 8;		 \
	  	SSPSTAT |= 0x00;     \
		DDRCbits.RC3 = 1;    \
	  	DDRCbits.RC4 = 1;    \
		SSPCON1 |= 0x20;  }   

// server flags
#define CFGBQ_DEFDLGONDISC	0x02
#define CFGBQ_VERSIONMASK	0x10
#define CFGBQ_VERSION3		0x10

// flexipanel protocol version number IDs used in message headers
#define FXPP_VERSION_2x 0x0001
#define FXPP_VERSION_3x 0x0002

// FlexiPanel data pointers 
#define FXPDATASTART 0x010000
#define pDevD ((rom bqFxPData*) FXPDATASTART)
#define pDlgD ((rom bqFxPDlgData*) (FXPDATASTART + sizeof(bqFxPData) ) )
#define pCtlD ((rom bqFxPCtlData*) (FXPDATASTART + pDevD->CtlOrg) )

typedef struct tagbqFxPData
{
//----- dword boundary 0x00
	unsigned char   nDialog;				// total number of controls or FF if uninitialized
	unsigned char	ConfigFlags;			// CFGBQ_ configuration flags used by flexipanel service				
	unsigned short	nCtl;					// total number of controls on all dialogs
//----- dword boundary 0x04
	unsigned short	RTCCtlIdx;				// Index of RTC contol, or 0XFFFF for none
	unsigned char	PingSecs;				// ping frequency, or 0 for no pings
	unsigned char	AppSpecificFlags;		// application specific flags set in Designer.exe
//----- dword boundary 0x08
	unsigned short	ProfileLoc;				// location of profile data in rom, offset from FXPDATASTART
	unsigned short	nProfile;				// number of profile entires
//----- dword boundary 0x0C
	unsigned short	CtlOrg;					// address of start of bqFxPCtlData stack in rom, offset from FXPDATASTART
	unsigned short	ServerFlags;			// MTF_ server flags sent to client (defined in hopcodes.h)
//----- dword boundary 0x10
	ShortHopHeader	SHopHead;				// server message header less last four bytes which vary from message to message
//----- dword boundary 0x1C
}  bqFxPData;


typedef struct tagbqFxPDlgData
{
//----- dword boundary 0x00
	unsigned short	firstCtl;					// index of first control bqFxPCtlData of this dialog
	unsigned char	nControl;					// number of controls in this dialog
	unsigned char	dummy1;
//----- dword boundary 0x04
}  bqFxPDlgData;


// ctl ID = Ctl_index + (0x100 * Dlg_index) + 1
// control types
#define TP_CtlTypeLatch 'G'
#define TP_CtlTypeNumber 'N'
#define TP_CtlTypeText 'T'
#define TP_CtlTypeButton 'B'
#define TP_CtlTypeSection 'S'
#define TP_CtlTypeImage 'I'
#define TP_CtlTypeDateTime 'D'
#define TP_CtlTypeMatrix 'M'
#define TP_CtlTypePassword 'P'
#define TP_CtlTypeList 'L'
#define TP_CtlTypeMessage 'X'
#define TP_CtlTypeBlob 'O'
#define TP_CtlTypeFile 'F'

typedef struct tagbqFxPCtlData
{
//----- dword boundary 0x00
	unsigned short	FixLoc;						// location of descriptor data in memory
	unsigned short	VarLoc;						// location of control value in memory				
//----- dword boundary 0x04
	unsigned short	IniLoc;						// location of initialization value in memory if control value is volatile storage; controls may tile the init data to save space
												// also where password & files controls store nonvolatile data - contact us for format details			
	unsigned short	FixLen;						// length of descriptor data in memory
//----- dword boundary 0x08
	unsigned short	VarLen;						// length of control value in memory			
	unsigned short	IniLen;						// length of initialization value in memory	
//----- dword boundary 0x0C
	unsigned long	dwflags;					// control flags
//----- dword boundary 0x10
	unsigned char	FixStr;						// STR_ storage location for fixed data
	unsigned char	VarStr;						// STR_ storage location for control value data 
	unsigned char	IniStr;						// STR_ storage location for initialization data
	unsigned char	Type;						// control type (upper byte assumed to be zero) 
//----- dword boundary 0x14
	unsigned short	VisDependentOn;				// dialog/control index on which the visibility of this control depends (for password unlocking), or 0xFFFF otherwise
	unsigned short	ID;							// control ID
//----- dword boundary 0x18
	unsigned char	nCtlVal;					// for matrix control, number of columns; for section contol, number of sub controls
	unsigned char 	dummy1;
	unsigned char 	dummy2;
	unsigned char 	dummy3;
//----- dword boundary 0x1C
}  bqFxPCtlData;

//-------------------------------------------------------------------------------------------------------

// Trace routines

#define SetupRAMTrace( pStart ) static unsigned char * pRAMTrace = pStart;\
								static unsigned char RAMTracePos = 0;

#define RAMTrace( TraceVal )	{\
								pRAMTrace[ RAMTracePos ] = TraceVal;\
								RAMTracePos++;\
								}

#define FillRAMTraceMem( FillVal )	{\
									RAMTracePos = 0;\
									do	{\
										pRAMTrace[ RAMTracePos ] = FillVal;\
										RAMTracePos++;\
										}\
									while (RAMTracePos);\
									}

// EETrace stores last 256 ee trace bytes in the 256 bytes from pStart+1
// Last offset EETracePos written is stored in pStart
#define SetupEETrace( pStartPos ) \
        rom unsigned short pStart = pStartPos; \
        static unsigned char EETracePos = 0; 

#define EELog( Addr, Data ) { \
		unsigned char tempint = 0;\
		/* suspend LinkMatik & CCP1*/  \
		if (LMTCTS==LMTCTSClear) tempint |= 0x04;   \
		LMTCTS = LMTCTSStop;   \
		if (CCP1Pin == RTSClear) tempint |= 0x08;   \
		CCP1Pin = RTSStop;   \
		msDelay( 1 ); \
		/* Set up write data */  \
		EEADR = (Addr);   \
		EEADRH = (Addr) >> 8;  \
		EEDATA = Data;  \
		EECON1bits.EEPGD = 0;  \
		EECON1bits.CFGS = 0;  \
		/* Disable interrupts & set write permissions */  \
		EECON1bits.WREN = 1;  \
		if (INTCONbits.GIEH) tempint |= 0x01;  \
		INTCONbits.GIEH = 0;  \
		if (INTCONbits.GIEL) tempint |= 0x02;  \
		INTCONbits.GIEL = 0;  \
		/*  Write data  */  \
		EECON2 = 0x55;  \
		EECON2 = 0xAA;  \
		EECON1bits.WR = 1;  \
		/* wait for write cycle to complete  */  \
		while (EECON1bits.WR == 1);  \
		/* Enable interrupts & disable eprom programming */  \
		if (tempint&0x01) INTCONbits.GIEH = 1;  \
		if (tempint&0x02) INTCONbits.GIEL = 1;  \
		if (tempint&0x04) LMTCTS = LMTCTSClear;   \
		if (tempint&0x08) CCP1Pin = RTSClear;   \
		EECON1bits.WREN = 0;  \
		}

#define EETrace( TraceVal )	{\
								EELog( pStart + EETracePos +1, TraceVal ); \
								EELog( pStart, EETracePos+1 ); \
								EETracePos++;\
								}

#define FillEETraceMem( FillVal )	{\
									EETracePos = 0;\
									do	{\
											EELog( pStart + EETracePos +1, FillVal ); \
											EETracePos++;\
										}\
									while (EETracePos);\
									EELog( pStart, EETracePos ); \
									}

//-------------------------------------------------------------------------------------------------------

// comparator functions

// comparators off
#define CmpsOff  { CMCON = 0b00000111; CVRCON = 0; }

// comparators set as inverters / buffers
// AN7 outputs the digital inverse of input AN11
// AN6 outputs the digital inverse of input AN9
// sets AN10 as a 2.5V output  
#define CmpsInvert  {\
				DirAN6 = 0;  \
				DirAN7 = 0;  \
				DirAN9 = 1;  \
				DirAN10 = 1;  \
				DirAN11 = 1;  \
				CMCON = 0b00000101; \
				CVRCON = 0b11101000;  \
				}

#define CmpsBuffer  {\
				DirAN6 = 0;  \
				DirAN7 = 0;  \
				DirAN9 = 1;  \
				DirAN10 = 1;  \
				DirAN11 = 1;  \
				CMCON = 0b00110101; \
				CVRCON = 0b11101000;  \
				}

// set comparators to AN9 and AN11 generate interrupts on state change
// the interrupt generated will be a PIR2bits.CMIF which must be cleared in software
// uses voltage reference set at 2.5 V
// set iPriority to 1 for high interrupt, 0 for low interrupt
// THIS IS UNTESTED
#define SetCmpsIntOnChange( iPriority )	{ \
				DirAN9 = 1;  \
				DirAN11 = 1;  \
				CMCON = 0b00000110;  \
				CVRCON = 0b10101000;  \
				IPR2bits.CMIP = iPriority;  \
				PIR2bits.CMIF = 0;  \
				PIE2bits.CMIE = 1;  \
				}


//  Serial UART
#define UARTRxD			PORTGbits.RG2  			/*  Serial Input to PIC */ 
#define	DirUARTRxD		TRISGbits.TRISG2
#define UARTTxD			PORTGbits.RG1  			/*  Serial Output from PIC */ 
#define	DirUARTTxD		TRISGbits.TRISG1
#define UARTTxDInit		1

//-------------------------------------------------------------------------------------------------------

// baud rates & serial

#ifdef TP_20MHz
#define SPBaudBrghZero1220		255  // use in conjuction with BRGHighVal = 0
#define SPBaudBrghZero2400		129  // use in conjuction with BRGHighVal = 0  	
#define SPBaudBrghZero4800		64	 // use in conjuction with BRGHighVal = 0	
#define SPBaudBrghOne9600		129  // use in conjuction with BRGHighVal = 1
#define SPBaudBrghOne19200		64	 // use in conjuction with BRGHighVal = 1
#define SPBaudBrghOne38400		32	 // use in conjuction with BRGHighVal = 1
#define SPBaudBrghOne57600		21	 // use in conjuction with BRGHighVal = 1
#define SPBaudBrghOne115200		10	 // use in conjuction with BRGHighVal = 1
#endif

#ifdef TP_24MHz
#define SPBaudBrghZero1220		"error you can't set this it's not possible"
#define SPBaudBrghZero2400		155  // use in conjuction with BRGHighVal = 0  	
#define SPBaudBrghZero4800		77	 // use in conjuction with BRGHighVal = 0	
#define SPBaudBrghOne9600		155  // use in conjuction with BRGHighVal = 1
#define SPBaudBrghOne19200		77	 // use in conjuction with BRGHighVal = 1
#define SPBaudBrghOne38400		38	 // use in conjuction with BRGHighVal = 1
#define SPBaudBrghOne57600		25	 // use in conjuction with BRGHighVal = 1
#define SPBaudBrghOne115200		12	 // use in conjuction with BRGHighVal = 1
#endif

#define InitUART( BRGVal, BRGHighVal )		{						\
							UARTTxD = UARTTxDInit;					\
							DirUARTTxD = DirOutput;					\
							DirUARTRxD = DirInput;  				\
						 	SPBRG2 = BRGVal;  						\
							TXSTA2bits.BRGH = BRGHighVal;			\
						    TXSTA2bits.SYNC = 0;  					\
						    TXSTA2bits.TX9 = 0; 					\
						    TXSTA2bits.TXEN = 1; 					\
						    RCSTA2bits.SPEN = 1; 					\
							RCSTA2bits.RX9 = 0; 					\
							IPR3bits.RC2IP = 1;    					\
							IPR3bits.TX2IP = 0;  					\
							PIE3bits.TX2IE = 0;	 					\
							PIE3bits.RC2IE = 1;						\
							RCSTA2bits.CREN = 0; 					\
							}

#define UARTFrameErr		RCSTA2bits.FERR
#define UARTOverrunErr		RCSTA2bits.OERR
#define UARTEnable			RCSTA2bits.CREN
#define UARTRxIntFlag		PIR3bits.RC2IF
#define UARTTxIntFlag		PIR3bits.TX2IF
#define UARTTxIntEnable		PIE3bits.TX2IE	 		
#define UARTIntAfterTx		PIE3bits.TX2IE = 1;	 		
#define UARTCnclIntAfterTx	PIE3bits.TX2IE = 0;	 		
#define UARTChIn			RCREG2
#define UARTChOut			TXREG2
#define UARTNotTxmitting	TXSTA2bits.TRMT
#define UARTRxIntEnable		PIE3bits.RC2IE

